--[[
	Description: Loot only what you want and keep your precious bag slots free of trash.
	Author: Eliote
--]]

local ADDON_NAME, PRIVATE_TABLE = ...;
local MOD_VERSION = GetAddOnMetadata(ADDON_NAME, "Version")

local L = PRIVATE_TABLE.GetTable("L")
local ConfigUI = PRIVATE_TABLE.GetTable("ConfigUI")
local AUTO_LOOTER = PRIVATE_TABLE.GetTable("AUTO_LOOTER")
local DataBase = PRIVATE_TABLE.GetTable("DB")
local Util = PRIVATE_TABLE.GetTable("Util")
local Color = PRIVATE_TABLE.GetTable("Color")

local print = Util.print

-- ADDON_LOADED
function AUTO_LOOTER:OnInitialize()
	--tinsert(UISpecialFrames, "AutoLooterFrame")

	local defaults = {
		profile = {
			items = {},
			ignore = {},
			alert = {},
			enable = true,
			printout = true,
			lootAll = false,
			lootQuest = true,
			close = false,
			minimapPos = 45,
			rarity = 2,
			price = 40000,
			alertSound = "Sound/creature/Murloc/mMurlocAggroOld.ogg",
			typeTable = AUTO_LOOTER.CreateAHTable()
		}
	}
	self.db = LibStub("AceDB-3.0"):New("AutoLooterDB", defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "ReloadOptions")
	self.db.RegisterCallback(self, "OnProfileCopied", "ReloadOptions")
	self.db.RegisterCallback(self, "OnProfileReset", "ReloadOptions")

	AUTO_LOOTER:ReloadOptions()

	AUTO_LOOTER:CreateProfile()

	DEFAULT_CHAT_FRAME:AddMessage(Color.BLUE .. "AutoLooter" .. Color.WHITE .. " || v" .. MOD_VERSION .. " || " .. Color.YELLOW .. "/autolooter /al /atl|r")
end

function AUTO_LOOTER:ReloadOptions()
	PRIVATE_TABLE.DB = self.db.profile
	DataBase = PRIVATE_TABLE.DB

	AUTO_LOOTER.Enable(DataBase.enable)

	AutoLooter_MinimapButton_Reposition()
end

-- LOOT OPENED
function AUTO_LOOTER:LOOT_OPENED(event, arg1)
	-- do not continue if autolooting
	if (arg1 == 1) then return end

	local srMoney = ""
	local srListed = ""
	local srRarity = ""
	local srPrice = ""
	local srAll = ""
	local srIgnore = ""
	local srType = ""
	local srQuest = ""
	local srToken = ""

	local questItemList = AUTO_LOOTER.CreateQuestItemList()

	for nIndex = 1, GetNumLootItems() do
		local _, sTitle, nQuantity, nRarity = GetLootSlotInfo(nIndex)
		local sItemLink = GetLootSlotLink(nIndex)

		-- Money
		if (nQuantity == 0) then
			srMoney = sTitle
			srMoney = string.gsub(srMoney, "\n", " ")
			LootSlot(nIndex)

			-- White List
		elseif (DataBase.items[sTitle]) then
			srListed = srListed .. sItemLink
			AUTO_LOOTER.Loot(nIndex, sTitle)

			-- Ignore List
		elseif (DataBase.ignore[sTitle]) then
			srIgnore = srIgnore .. sItemLink

			-- Rarity
		elseif (DataBase.rarity > -1) and nRarity and (nRarity >= DataBase.rarity) then
			srRarity = srRarity .. sItemLink
			AUTO_LOOTER.Loot(nIndex, sTitle)

			-- Need more info
		elseif sItemLink then
			local _, _, _, _, _, itemType, itemSubType, _, equipSlot, _, iPrice = GetItemInfo(sItemLink)

			-- Type/Subtype
			if AUTO_LOOTER.LootType(itemType, itemSubType) then
				srType = srType .. Color.YELLOW .. "(" .. itemType .. "/" .. itemSubType .. ")|r" .. sItemLink
				AUTO_LOOTER.Loot(nIndex, sTitle)

				-- Container
			elseif questItemList and questItemList[sTitle] then
				srQuest = srQuest .. sItemLink
				AUTO_LOOTER.Loot(nIndex, sTitle)

				-- Token
			elseif not iPrice then
				srToken = srToken .. sItemLink
				AUTO_LOOTER.Loot(nIndex, sTitle)

				-- Price
			elseif (tonumber(DataBase.price) > 0) and (tonumber(iPrice) >= tonumber(DataBase.price)) then
				srPrice = srPrice .. sItemLink
				AUTO_LOOTER.Loot(nIndex, sTitle)

				-- LootAll
			elseif (DataBase.lootAll) then
				srAll = srAll .. sItemLink
				AUTO_LOOTER.Loot(nIndex, sTitle)
			end
		end
	end

	if (DataBase.close) then
		CloseLoot()
	end

	if (DataBase.printout) then
		local sResult = ""

		if (srMoney ~= "") then sResult = sResult .. Color.GREEN .. L["Coin"] .. "|r:|cffffbb44[" .. srMoney .. "]|r  " end
		if (srListed ~= "") then sResult = sResult .. Color.GREEN .. L["Listed"] .. "|r:" .. srListed .. "  " end
		if (srRarity ~= "") then sResult = sResult .. Color.GREEN .. L["Rarity"] .. "|r:" .. srRarity .. "  " end
		if (srPrice ~= "") then sResult = sResult .. Color.GREEN .. L["Price"] .. "|r:" .. srPrice .. "  " end
		if (srAll ~= "") then sResult = sResult .. Color.GREEN .. L["All"] .. "|r:" .. srAll .. " " end
		if (srIgnore ~= "") then sResult = sResult .. Color.GREEN .. L["Ignored"] .. "|r:" .. srIgnore .. " " end
		if (srType ~= "") then sResult = sResult .. Color.GREEN .. L["Type"] .. "|r:" .. srType .. " " end
		if (srQuest ~= "") then sResult = sResult .. Color.GREEN .. L["Quest"] .. "|r:" .. srQuest .. " " end
		if (srToken ~= "") then sResult = sResult .. Color.GREEN .. L["Token"] .. "|r:" .. srToken .. " " end

		if (sResult ~= "") then print(sResult) end
	end
end

function AUTO_LOOTER.LootType(iType, iSubType)
	local t = DataBase.typeTable[iType]
	if t then return t[iSubType] end

	return false
end

function AUTO_LOOTER.CreateQuestItemList()
	if not DataBase.lootQuest then return end

	local itemList = {}

	for questIndex = 1, GetNumQuestLogEntries() do
		for boardIndex = 1, GetNumQuestLeaderBoards(questIndex) do
			local leaderboardTxt, boardItemType, isDone = GetQuestLogLeaderBoard(boardIndex, questIndex)

			if not isDone and boardItemType == "item" then
				local i, j, numItems, numNeeded, itemName = string.find(leaderboardTxt, "([%d]+)%s*/%s*([%d]+)%s*(.*)%s*")

				if itemName then
					itemList[itemName] = true
				end
			end
		end
	end

	return itemList
end

function AUTO_LOOTER:CreateProfile()
	self.options = {
		type = "group",
		name = "AutoLooter",
		args = {
			add = {
				type = "input",
				name = L["Add item to white list"],
				set = function(info, val) AUTO_LOOTER.AddItem(val, DataBase.items) end,
				get = false,
				usage = L["[link/id/name] or [mouse over]"]
			},
			ignore = {
				type = "input",
				name = L["Add item to ignore list"],
				set = function(info, val) AUTO_LOOTER.AddItem(val, DataBase.ignore) end,
				get = false
			},
			alert = {
				type = "input",
				name = L["Add item to alert list"],
				set = function(info, val) AUTO_LOOTER.AddItem(val, DataBase.alert) end,
				get = false
			},
			remove = {
				type = "input",
				name = L["Remove item from white list"],
				set = function(info, val) AUTO_LOOTER.RemoveItem(val, DataBase.items) end,
				get = false
			},
			removeI = {
				type = "input",
				name = L["Remove item from ignore list"],
				set = function(info, val) AUTO_LOOTER.RemoveItem(val, DataBase.ignore) end,
				get = false
			},
			removeA = {
				type = "input",
				name = L["Remove item from alert list"],
				set = function(info, val) AUTO_LOOTER.RemoveItem(val, DataBase.alert) end,
				get = false
			},
			enable = {
				type = "toggle",
				name = L["Enable"],
				--desc = L["Enable/Disable the addon"],
				set = function(info, val) AUTO_LOOTER.Enable(val) end,
				get = function(info) return DataBase.enable end
			},
			printout = {
				type = "toggle",
				name = L["Printout items looted"],
				set = function(info, val) DataBase.printout = AUTO_LOOTER.GetBoolean(val) end,
				get = function(info) return DataBase.printout end
			},
			lootAll = {
				type = "toggle",
				name = L["Loot everything"],
				set = function(info, val) DataBase.lootAll = AUTO_LOOTER.GetBoolean(val) end,
				get = function(info) return DataBase.lootAll end
			},
			lootQuest = {
				type = "toggle",
				name = L["Loot quest itens"],
				set = function(info, val) DataBase.lootQuest = AUTO_LOOTER.GetBoolean(val) end,
				get = function(info) return DataBase.lootQuest end
			},
			close = {
				type = "toggle",
				name = L["Close after loot"],
				set = function(info, val) DataBase.close = AUTO_LOOTER.GetBoolean(val) end,
				get = function(info) return DataBase.close end
			},
			rarity = {
				type = "select",
				name = L["Rarity"],
				values = ConfigUI.raritysMenu,
				set = function(info, val) DataBase.rarity = val end,
				get = function(info) return DataBase.rarity end
			},
			price = {
				type = "range",
				name = L["Price (in coppers)"],
				min = 0,
				max = 10000000,
				step = 1,
				set = function(info, val) DataBase.price = val end,
				get = function(info) return DataBase.price end
			}
		}
	}

	local AceConfig = LibStub("AceConfig-3.0")
	AceConfig:RegisterOptionsTable("AutoLooter-Commands", self.options, { "al", "autolooter", "atl" })
	AceConfig:RegisterOptionsTable("AutoLooter", { type = "group", name = "AutoLooter", args = { enable = self.options.args.enable } })
	AceConfig:RegisterOptionsTable("AutoLooter-Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db))

	local AceDialog = LibStub("AceConfigDialog-3.0")
	self.optionsFrame = AceDialog:AddToBlizOptions("AutoLooter", "AutoLooter")
	self.profilesFrame = AceDialog:AddToBlizOptions("AutoLooter-Profiles", "Profiles", "AutoLooter")
end

function AUTO_LOOTER:CreateAHTable(defTable)
	local out = {}

	local itemClasses = { GetAuctionItemClasses() };
	if #itemClasses > 0 then
		for i, itemClass in pairs(itemClasses) do
			local t = {}

			local itemSubClasses = { GetAuctionItemSubClasses(i) };
			if #itemSubClasses > 0 then
				for j, itemSubClass in pairs(itemSubClasses) do
					t[itemSubClass] = AUTO_LOOTER.GetSaved(defTable, itemClass, itemSubClass)
				end
			else
				t[itemClass] = AUTO_LOOTER.GetSaved(defTable, itemClass, itemClass)
			end

			out[itemClass] = t
		end
	end

	return out
end

function AUTO_LOOTER.GetSaved(defTable, itemClass, itemSubClass)
	if not defTable then return false end

	local t = defTable[itemClass]
	if t then return t[itemSubClass] end
end

-- FUNCTIONS

function AUTO_LOOTER.GetBoolean(bool, def)
	if (bool == "on" or bool == true) then
		return true
	elseif (bool == "off" or bool == false) then
		return false
	end

	if not def then return false end

	return def
end

function AUTO_LOOTER.Loot(index, itemName)
	AUTO_LOOTER.DoAlert(itemName, index)
	LootSlot(index)
	ConfirmLootSlot(index) -- In case it's a Bind on Pickup
end

function AUTO_LOOTER.Enable(bool)
	DataBase.enable = AUTO_LOOTER.GetBoolean(bool, not DataBase.enable)

	if DataBase.enable then
		AUTO_LOOTER:RegisterEvent("LOOT_OPENED")
		DataBase.enable = true
		print(L["Enabled"])
	else
		AUTO_LOOTER:UnregisterEvent("LOOT_OPENED")
		DataBase.enable = false
		print(L["Disabled"])
	end
end

function AUTO_LOOTER.DoAlert(sTitle, nIndex)
	if DataBase.alert[sTitle] then
		PlaySoundFile(DataBase.alertSound)
		RaidNotice_AddMessage(RaidWarningFrame, GetLootSlotLink(nIndex), ChatTypeInfo["RAID_WARNING"])
	end
end

function AUTO_LOOTER.AddItem(sTitle, list)
	if not sTitle or sTitle == "" then
		sTitle = GameTooltip:GetItem()

		if not sTitle or sTitle == "" then
			for k, v in pairs(list) do
				print(k)
			end

			return
		end
	end

	local sName = GetItemInfo(sTitle)

	if not sName then print(L["Invalid item"], ": ", sTitle); return end

	if (list[sName]) then
		print(L["Already in the list"], ": ", Color.YELLOW, sName)
		return
	end

	list[sName] = true
	print(L["Added"], ": ", Color.YELLOW, sName)
end

function AUTO_LOOTER.RemoveItem(sTitle, list)
	if not sTitle or sTitle == "" then
		sTitle = GameTooltip:GetItem()

		if not sTitle or sTitle == "" then
			for k, v in pairs(list) do
				print(k)
			end

			return
		end
	end

	local sName = GetItemInfo(sTitle)

	if (list[sName]) then
		list[sName] = nil
		print(L["Removed"], ": ", Color.YELLOW, sName)
		return
	end

	print(L["Not listed"], ": ", Color.YELLOW, sName)
end
